@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * GData Client
- * Copyright (C) Philip Withnall 2009 <philip@tecnocode.co.uk>
+ * Copyright (C) Philip Withnall 2009–2010 <philip@tecnocode.co.uk>
  *
  * GData Client is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,7 +24,7 @@
  * @include: gdata/gd/gdata-gd-email-address.h
  *
  * #GDataGDEmailAddress represents an "email" element from the
- * <ulink type="http" url="http://code.google.com/apis/gdata/docs/1.0/elements.html#gdEmail">GData specification</ulink>.
+ * <ulink type="http" url="http://code.google.com/apis/gdata/docs/2.0/elements.html#gdEmail">GData specification</ulink>.
  **/
 
 #include <glib.h>
@@ -46,13 +46,15 @@ struct _GDataGDEmailAddressPrivate {
 	gchar *relation_type;
 	gchar *label;
 	gboolean is_primary;
+	gchar *display_name;
 };
 
 enum {
 	PROP_ADDRESS = 1,
 	PROP_RELATION_TYPE,
 	PROP_LABEL,
-	PROP_IS_PRIMARY
+	PROP_IS_PRIMARY,
+	PROP_DISPLAY_NAME
 };
 
 G_DEFINE_TYPE (GDataGDEmailAddress, gdata_gd_email_address, GDATA_TYPE_PARSABLE)
@@ -82,7 +84,7 @@ gdata_gd_email_address_class_init (GDataGDEmailAddressClass *klass)
 	 * The e-mail address itself.
 	 *
 	 * For more information, see the
-	 * <ulink type="http" url="http://code.google.com/apis/gdata/docs/1.0/elements.html#gdEmail">GData specification</ulink>.
+	 * <ulink type="http" url="http://code.google.com/apis/gdata/docs/2.0/elements.html#gdEmail">GData specification</ulink>.
 	 *
 	 * Since: 0.4.0
 	 **/
@@ -98,7 +100,7 @@ gdata_gd_email_address_class_init (GDataGDEmailAddressClass *klass)
 	 * A programmatic value that identifies the type of e-mail address.
 	 *
 	 * For more information, see the
-	 * <ulink type="http" url="http://code.google.com/apis/gdata/docs/1.0/elements.html#gdEmail">GData specification</ulink>.
+	 * <ulink type="http" url="http://code.google.com/apis/gdata/docs/2.0/elements.html#gdEmail">GData specification</ulink>.
 	 *
 	 * Since: 0.4.0
 	 **/
@@ -114,7 +116,7 @@ gdata_gd_email_address_class_init (GDataGDEmailAddressClass *klass)
 	 * A simple string value used to name this e-mail address. It allows UIs to display a label such as "Work", "Personal", "Preferred", etc.
 	 *
 	 * For more information, see the
-	 * <ulink type="http" url="http://code.google.com/apis/gdata/docs/1.0/elements.html#gdEmail">GData specification</ulink>.
+	 * <ulink type="http" url="http://code.google.com/apis/gdata/docs/2.0/elements.html#gdEmail">GData specification</ulink>.
 	 *
 	 * Since: 0.4.0
 	 **/
@@ -130,7 +132,7 @@ gdata_gd_email_address_class_init (GDataGDEmailAddressClass *klass)
 	 * Indicates which e-mail address out of a group is primary.
 	 *
 	 * For more information, see the
-	 * <ulink type="http" url="http://code.google.com/apis/gdata/docs/1.0/elements.html#gdEmail">GData specification</ulink>.
+	 * <ulink type="http" url="http://code.google.com/apis/gdata/docs/2.0/elements.html#gdEmail">GData specification</ulink>.
 	 *
 	 * Since: 0.4.0
 	 **/
@@ -139,6 +141,22 @@ gdata_gd_email_address_class_init (GDataGDEmailAddressClass *klass)
 					"Primary?", "Indicates which e-mail address out of a group is primary.",
 					FALSE,
 					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+	/**
+	 * GDataGDEmailAddress:display-name:
+	 *
+	 * A display name of the entity (e.g. a person) the e-mail address belongs to.
+	 *
+	 * For more information, see the
+	 * <ulink type="http" url="http://code.google.com/apis/gdata/docs/2.0/elements.html#gdEmail">GData specification</ulink>.
+	 *
+	 * Since: 0.6.0
+	 **/
+	g_object_class_install_property (gobject_class, PROP_DISPLAY_NAME,
+				g_param_spec_string ("display-name",
+					"Display name", "A display name of the entity (e.g. a person) the e-mail address belongs to.",
+					NULL,
+					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -155,6 +173,7 @@ gdata_gd_email_address_finalize (GObject *object)
 	g_free (priv->address);
 	g_free (priv->relation_type);
 	g_free (priv->label);
+	g_free (priv->display_name);
 
 	/* Chain up to the parent class */
 	G_OBJECT_CLASS (gdata_gd_email_address_parent_class)->finalize (object);
@@ -178,6 +197,9 @@ gdata_gd_email_address_get_property (GObject *object, guint property_id, GValue
 		case PROP_IS_PRIMARY:
 			g_value_set_boolean (value, priv->is_primary);
 			break;
+		case PROP_DISPLAY_NAME:
+			g_value_set_string (value, priv->display_name);
+			break;
 		default:
 			/* We don't have any other property... */
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -203,6 +225,9 @@ gdata_gd_email_address_set_property (GObject *object, guint property_id, const G
 		case PROP_IS_PRIMARY:
 			gdata_gd_email_address_set_is_primary (self, g_value_get_boolean (value));
 			break;
+		case PROP_DISPLAY_NAME:
+			gdata_gd_email_address_set_display_name (self, g_value_get_string (value));
+			break;
 		default:
 			/* We don't have any other property... */
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -213,7 +238,7 @@ gdata_gd_email_address_set_property (GObject *object, guint property_id, const G
 static gboolean
 pre_parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *root_node, gpointer user_data, GError **error)
 {
-	xmlChar *address, *rel, *label, *primary;
+	xmlChar *address, *rel, *primary;
 	gboolean primary_bool;
 	GDataGDEmailAddressPrivate *priv = GDATA_GD_EMAIL_ADDRESS (parsable)->priv;
 
@@ -242,17 +267,11 @@ pre_parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *root_node, gpointe
 	}
 	xmlFree (primary);
 
-	/* Other properties */
-	label = xmlGetProp (root_node, (xmlChar*) "label");
-
-	priv->address = g_strdup ((gchar*) address);
-	priv->relation_type = g_strdup ((gchar*) rel);
-	priv->label = g_strdup ((gchar*) label);
+	priv->address = (gchar*) address;
+	priv->relation_type = (gchar*) rel;
+	priv->label = (gchar*) xmlGetProp (root_node, (xmlChar*) "label");
 	priv->is_primary = primary_bool;
-
-	xmlFree (address);
-	xmlFree (rel);
-	xmlFree (label);
+	priv->display_name = (gchar*) xmlGetProp (root_node, (xmlChar*) "displayName");
 
 	return TRUE;
 }
@@ -267,6 +286,8 @@ pre_get_xml (GDataParsable *parsable, GString *xml_string)
 		g_string_append_printf (xml_string, " rel='%s'", priv->relation_type);
 	if (priv->label != NULL)
 		gdata_parser_string_append_escaped (xml_string, " label='", priv->label, "'");
+	if (priv->display_name != NULL)
+		gdata_parser_string_append_escaped (xml_string, " displayName='", priv->display_name, "'");
 
 	if (priv->is_primary == TRUE)
 		g_string_append (xml_string, " primary='true'");
@@ -288,7 +309,7 @@ get_namespaces (GDataParsable *parsable, GHashTable *namespaces)
  * @is_primary: %TRUE if this e-mail address is its owner's primary address, %FALSE otherwise
  *
  * Creates a new #GDataGDEmailAddress. More information is available in the <ulink type="http"
- * url="http://code.google.com/apis/gdata/docs/1.0/elements.html#gdEmail">GData specification</ulink>.
+ * url="http://code.google.com/apis/gdata/docs/2.0/elements.html#gdEmail">GData specification</ulink>.
  *
  * Return value: a new #GDataGDEmailAddress, or %NULL; unref with g_object_unref()
  *
@@ -478,3 +499,41 @@ gdata_gd_email_address_set_is_primary (GDataGDEmailAddress *self, gboolean is_pr
 	self->priv->is_primary = is_primary;
 	g_object_notify (G_OBJECT (self), "is-primary");
 }
+
+/**
+ * gdata_gd_email_address_get_display_name:
+ * @self: a #GDataGDEmailAddress
+ *
+ * Gets the #GDataGDEmailAddress:display-name property.
+ *
+ * Return value: a display name for the e-mail address, or %NULL
+ *
+ * Since: 0.6.0
+ **/
+const gchar *
+gdata_gd_email_address_get_display_name (GDataGDEmailAddress *self)
+{
+	g_return_val_if_fail (GDATA_IS_GD_EMAIL_ADDRESS (self), NULL);
+	return self->priv->display_name;
+}
+
+/**
+ * gdata_gd_email_address_set_display_name:
+ * @self: a #GDataGDEmailAddress
+ * @display_name: the new display name, or %NULL
+ *
+ * Sets the #GDataGDEmailAddress:display-name property to @display_name.
+ *
+ * Set @display_name to %NULL to unset the property in the e-mail address.
+ *
+ * Since: 0.6.0
+ **/
+void
+gdata_gd_email_address_set_display_name (GDataGDEmailAddress *self, const gchar *display_name)
+{
+	g_return_if_fail (GDATA_IS_GD_EMAIL_ADDRESS (self));
+
+	g_free (self->priv->display_name);
+	self->priv->display_name = g_strdup (display_name);
+	g_object_notify (G_OBJECT (self), "display-name");
+}
