@@ -37,6 +37,7 @@
 
 #include "gdata-youtube-query.h"
 #include "gdata-query.h"
+#include "gdata-youtube-content.h"
 
 static void gdata_youtube_query_finalize (GObject *object);
 static void gdata_youtube_query_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
@@ -93,7 +94,7 @@ gdata_youtube_query_class_init (GDataYouTubeQueryClass *klass)
 	/**
 	 * GDataYouTubeQuery:format:
 	 *
-	 * Specifies that videos must be available in a particular video format. Use %GDATA_YOUTUBE_FORMAT_ALL to
+	 * Specifies that videos must be available in a particular video format. Use %GDATA_YOUTUBE_FORMAT_UNKNOWN to
 	 * retrieve videos irrespective of their format availability.
 	 *
 	 * Since: 0.3.0
@@ -101,7 +102,7 @@ gdata_youtube_query_class_init (GDataYouTubeQueryClass *klass)
 	g_object_class_install_property (gobject_class, PROP_FORMAT,
 				g_param_spec_enum ("format",
 					"Format", "Specifies that videos must be available in a particular video format.",
-					GDATA_TYPE_YOUTUBE_FORMAT, GDATA_YOUTUBE_FORMAT_ALL,
+					GDATA_TYPE_YOUTUBE_FORMAT, GDATA_YOUTUBE_FORMAT_UNKNOWN,
 					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 	/**
@@ -377,19 +378,15 @@ gdata_youtube_query_set_property (GObject *object, guint property_id, const GVal
 			break;
 		case PROP_LATITUDE:
 			self->priv->latitude = g_value_get_double (value);
-			g_object_notify (object, "latitude");
 			break;
 		case PROP_LONGITUDE:
 			self->priv->longitude = g_value_get_double (value);
-			g_object_notify (object, "longitude");
 			break;
 		case PROP_LOCATION_RADIUS:
 			self->priv->location_radius = g_value_get_double (value);
-			g_object_notify (object, "location-radius");
 			break;
 		case PROP_HAS_LOCATION:
 			self->priv->has_location = g_value_get_boolean (value);
-			g_object_notify (object, "has-location");
 			break;
 		case PROP_LANGUAGE:
 			gdata_youtube_query_set_language (self, g_value_get_string (value));
@@ -462,33 +459,38 @@ get_query_uri (GDataQuery *self, const gchar *feed_uri, GString *query_uri, gboo
 			g_assert_not_reached ();
 	}
 
-	if (priv->format != GDATA_YOUTUBE_FORMAT_ALL)
+	if (priv->format != GDATA_YOUTUBE_FORMAT_UNKNOWN)
 		g_string_append_printf (query_uri, "&format=%u", priv->format);
 
 	if (priv->latitude >= -90.0 && priv->latitude <= 90.0 &&
 	    priv->longitude >= -180.0 && priv->longitude <= 180.0) {
-		g_string_append_printf (query_uri, (priv->has_location == TRUE) ? "&location=%f,%f!" : "&location=%f,%f",
-					priv->latitude, priv->longitude);
+		gchar latitude[G_ASCII_DTOSTR_BUF_SIZE], longitude[G_ASCII_DTOSTR_BUF_SIZE];
 
-		if (priv->location_radius >= 0.0)
-			g_string_append_printf (query_uri, "&location-radius=%fm", priv->location_radius);
+		g_string_append_printf (query_uri, (priv->has_location == TRUE) ? "&location=%s,%s!" : "&location=%s,%s",
+					g_ascii_dtostr (latitude, sizeof (latitude), priv->latitude),
+					g_ascii_dtostr (longitude, sizeof (longitude), priv->longitude));
+
+		if (priv->location_radius >= 0.0) {
+			gchar radius[G_ASCII_DTOSTR_BUF_SIZE];
+			g_string_append_printf (query_uri, "&location-radius=%sm", g_ascii_dtostr (radius, sizeof (radius), priv->location_radius));
+		}
 	} else if (priv->has_location == TRUE) {
 		g_string_append (query_uri, "&location=!");
 	}
 
 	if (priv->language != NULL) {
 		g_string_append (query_uri, "&lr=");
-		g_string_append_uri_escaped (query_uri, priv->language, NULL, TRUE);
+		g_string_append_uri_escaped (query_uri, priv->language, NULL, FALSE);
 	}
 
 	if (priv->order_by != NULL) {
 		g_string_append (query_uri, "&orderby=");
-		g_string_append_uri_escaped (query_uri, priv->order_by, NULL, TRUE);
+		g_string_append_uri_escaped (query_uri, priv->order_by, NULL, FALSE);
 	}
 
 	if (priv->restriction != NULL) {
 		g_string_append (query_uri, "&restriction=");
-		g_string_append_uri_escaped (query_uri, priv->restriction, NULL, TRUE);
+		g_string_append_uri_escaped (query_uri, priv->restriction, NULL, FALSE);
 	}
 
 	if (priv->sort_order != GDATA_YOUTUBE_SORT_NONE) {
@@ -533,7 +535,7 @@ gdata_youtube_query_new (const gchar *q)
 GDataYouTubeFormat
 gdata_youtube_query_get_format (GDataYouTubeQuery *self)
 {
-	g_return_val_if_fail (GDATA_IS_YOUTUBE_QUERY (self), GDATA_YOUTUBE_FORMAT_ALL);
+	g_return_val_if_fail (GDATA_IS_YOUTUBE_QUERY (self), GDATA_YOUTUBE_FORMAT_UNKNOWN);
 	return self->priv->format;
 }
 
@@ -605,10 +607,12 @@ gdata_youtube_query_set_location (GDataYouTubeQuery *self, gdouble latitude, gdo
 	self->priv->location_radius = radius;
 	self->priv->has_location = has_location;
 
+	g_object_freeze_notify (G_OBJECT (self));
 	g_object_notify (G_OBJECT (self), "latitude");
 	g_object_notify (G_OBJECT (self), "longitude");
 	g_object_notify (G_OBJECT (self), "location-radius");
 	g_object_notify (G_OBJECT (self), "has-location");
+	g_object_thaw_notify (G_OBJECT (self));
 }
 
 /**
