@@ -19,10 +19,29 @@
 
 #include <glib.h>
 #include <unistd.h>
+#include <string.h>
 
 #include "gdata.h"
 #include "common.h"
 
+static void
+check_kind (GDataEntry *entry)
+{
+	GList *list;
+	gboolean has_kind = FALSE;
+
+	/* Check the contact's kind category is present and correct */
+	for (list = gdata_entry_get_categories (entry); list != NULL; list = list->next) {
+		GDataCategory *category = GDATA_CATEGORY (list->data);
+
+		if (strcmp (gdata_category_get_scheme (category), "http://schemas.google.com/g/2005#kind") == 0) {
+			g_assert_cmpstr (gdata_category_get_term (category), ==, "http://schemas.google.com/contact/2008#contact");
+			has_kind = TRUE;
+		}
+	}
+	g_assert (has_kind == TRUE);
+}
+
 static GDataContactsContact *
 get_contact (gconstpointer service)
 {
@@ -46,6 +65,7 @@ get_contact (gconstpointer service)
 	g_assert (entries != NULL);
 	entry = entries->data;
 	g_assert (GDATA_IS_CONTACTS_CONTACT (entry));
+	check_kind (entry);
 
 	g_object_ref (entry);
 	g_object_unref (feed);
@@ -95,7 +115,7 @@ test_query_all_contacts (gconstpointer service)
 	g_assert (GDATA_IS_FEED (feed));
 	g_clear_error (&error);
 
-	/* TODO: check entries and feed properties */
+	/* TODO: check entries, kinds and feed properties */
 
 	g_object_unref (feed);
 }
@@ -149,6 +169,10 @@ test_insert_simple (gconstpointer service)
 	contact = gdata_contacts_contact_new (NULL);
 	g_get_current_time (&creation_time);
 
+	/* Check the kind is present and correct */
+	g_assert (GDATA_IS_CONTACTS_CONTACT (contact));
+	check_kind (GDATA_ENTRY (contact));
+
 	/* Set and check the name (to check if the title of the entry is updated) */
 	gdata_entry_set_title (GDATA_ENTRY (contact), "Elizabeth Bennet");
 	name = gdata_contacts_contact_get_name (contact);
@@ -156,6 +180,12 @@ test_insert_simple (gconstpointer service)
 	gdata_gd_name_set_full_name (name, "Lizzie Bennet");
 	g_assert_cmpstr (gdata_entry_get_title (GDATA_ENTRY (contact)), ==, "Lizzie Bennet");
 
+	name2 = gdata_gd_name_new ("John", "Smith");
+	gdata_gd_name_set_full_name (name2, "John Smith");
+	gdata_contacts_contact_set_name (contact, name2);
+	g_object_unref (name2);
+	g_assert_cmpstr (gdata_entry_get_title (GDATA_ENTRY (contact)), ==, "John Smith");
+
 	gdata_entry_set_content (GDATA_ENTRY (contact), "Notes");
 	/* TODO: Have it add this category automatically? Same for GDataCalendarEvent */
 	category = gdata_category_new ("http://schemas.google.com/contact/2008#contact", "http://schemas.google.com/g/2005#kind", NULL);
@@ -192,7 +222,7 @@ test_insert_simple (gconstpointer service)
 	              "edited", &edited,
 	              "deleted", &deleted,
 	              "has-photo", &has_photo,
-	              "name", &name2,
+	              "name", &name,
 	              NULL);
 
 	g_assert_cmpint (edited->tv_sec, ==, creation_time.tv_sec);
@@ -207,13 +237,17 @@ test_insert_simple (gconstpointer service)
 	xml = gdata_parsable_get_xml (GDATA_PARSABLE (contact));
 	g_assert_cmpstr (xml, ==,
 			 "<entry xmlns='http://www.w3.org/2005/Atom' "
-			 	"xmlns:gd='http://schemas.google.com/g/2005' "
-			 	"xmlns:app='http://www.w3.org/2007/app' "
-			 	"xmlns:gContact='http://schemas.google.com/contact/2008'>"
-			 	"<title type='text'>Lizzie Bennet</title>"
-			 	"<content type='text'>Notes</content>"
+				"xmlns:gd='http://schemas.google.com/g/2005' "
+				"xmlns:app='http://www.w3.org/2007/app' "
+				"xmlns:gContact='http://schemas.google.com/contact/2008'>"
+				"<title type='text'>John Smith</title>"
+				"<content type='text'>Notes</content>"
 				"<category term='http://schemas.google.com/contact/2008#contact' scheme='http://schemas.google.com/g/2005#kind'/>"
-				"<gd:name><gd:fullName>Lizzie Bennet</gd:fullName></gd:name>"
+				"<gd:name>"
+					"<gd:givenName>John</gd:givenName>"
+					"<gd:familyName>Smith</gd:familyName>"
+					"<gd:fullName>John Smith</gd:fullName>"
+				"</gd:name>"
 				"<gd:email address='liz@gmail.com' rel='http://schemas.google.com/g/2005#work' primary='false'/>"
 				"<gd:email address='liz@example.org' rel='http://schemas.google.com/g/2005#home' primary='false'/>"
 				"<gd:im address='liz@gmail.com' protocol='http://schemas.google.com/g/2005#GOOGLE_TALK' "
@@ -231,6 +265,7 @@ test_insert_simple (gconstpointer service)
 	new_contact = gdata_contacts_service_insert_contact (GDATA_CONTACTS_SERVICE (service), contact, NULL, &error);
 	g_assert_no_error (error);
 	g_assert (GDATA_IS_CONTACTS_CONTACT (new_contact));
+	check_kind (GDATA_ENTRY (new_contact));
 	g_clear_error (&error);
 
 	/* Check its edited date */
@@ -403,7 +438,8 @@ test_parser_minimal (gconstpointer service)
 			"<gd:email rel='http://schemas.google.com/g/2005#other' address='bob@example.com'/>"
 		"</entry>", -1, &error));
 	g_assert_no_error (error);
-	g_assert (GDATA_IS_ENTRY (contact));
+	g_assert (GDATA_IS_CONTACTS_CONTACT (contact));
+	check_kind (GDATA_ENTRY (contact));
 	g_clear_error (&error);
 
 	/* Check the contact's properties */
@@ -443,7 +479,8 @@ test_parser_normal (gconstpointer service)
 			"<gd:deleted/>"
 		"</entry>", -1, &error));
 	g_assert_no_error (error);
-	g_assert (GDATA_IS_ENTRY (contact));
+	g_assert (GDATA_IS_CONTACTS_CONTACT (contact));
+	check_kind (GDATA_ENTRY (contact));
 	g_clear_error (&error);
 
 	/* TODO: Check the other properties */
@@ -518,7 +555,8 @@ test_photo_has_photo (gconstpointer service)
 				"href='http://www.google.com/m8/feeds/photos/media/libgdata.test@googlemail.com/1b46cdd20bfbee3b'/>"
 		"</entry>", -1, &error));
 	g_assert_no_error (error);
-	g_assert (GDATA_IS_ENTRY (contact));
+	g_assert (GDATA_IS_CONTACTS_CONTACT (contact));
+	check_kind (GDATA_ENTRY (contact));
 	g_clear_error (&error);
 
 	/* Check for no photo */
@@ -545,7 +583,8 @@ test_photo_has_photo (gconstpointer service)
 				"gd:etag='&quot;QngzcDVSLyp7ImA9WxJTFkoITgU.&quot;'/>"
 		"</entry>", -1, &error));
 	g_assert_no_error (error);
-	g_assert (GDATA_IS_ENTRY (contact));
+	g_assert (GDATA_IS_CONTACTS_CONTACT (contact));
+	check_kind (GDATA_ENTRY (contact));
 	g_clear_error (&error);
 
 	g_assert (gdata_contacts_contact_has_photo (contact) == TRUE);
